/* Copyright (C) 2019 Terence Kelly.  All rights reserved.
   Failure-atomic msync() in user space via snapshots:  Test.
*/

#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "famus_snap.h"

#define  P (void) printf
#define FP (void)fprintf

static int Open(const char *p, int f, mode_t m, int line) {
  int fd = open(p, f, m);
  if (0 > fd) {
    FP(stderr, "%d: 'open(\"%s\")' failed: %s\n", line, p, strerror(errno));
    exit(EXIT_FAILURE);
  }
  return fd;
}

#define OPEN(p, f, m) Open((p), (f), (m), __LINE__)

#define BAIL(msg, s, r)                       \
do {                                          \
  FP(stderr, "%d: %s, status %d: '%s'\n",     \
     __LINE__, (msg), (s), strerror(errno));  \
  return (r);                                 \
} while (0)

int main(int argc, char *argv[]) {
  int bfd, snapfd1, snapfd2, dirfd, s;
  char *p;
  size_t len;
  if (4 != argc) {
    FP(stderr, "%s  backingfile  snapshotfile1 snapshotfile2\n", argv[0]);
    return 1;
  }
  bfd     = OPEN(argv[1], O_RDWR,                      0      );
  snapfd1 = OPEN(argv[2], O_WRONLY | O_CREAT | O_EXCL, S_IWUSR);
  snapfd2 = OPEN(argv[3], O_WRONLY | O_CREAT | O_EXCL, S_IWUSR);
  dirfd   = OPEN(".",     O_RDONLY,                    0      );
  p = (char *)famus_snap_map(NULL, &len, MAP_SHARED, bfd, snapfd1, dirfd, &s);
  if (NULL == p)
    BAIL("'famus_snap_map()' failed", s, 2);
  assert(0 == s);
  *p = 'b';
  s = famus_snap_sync(bfd, snapfd2, 0);
  if (0 != s)
    BAIL("'famus_snap_sync()' failed", s, 3);
  s = munmap(p, len);
  if (0 != s)
    BAIL("'munmap() failed'", s, 4);
  return 0;
}

